# Projects
_Projects_ are the core organisational feature of Treadl. Think of them like folders on your computer - you can use them to store images, patterns, and other arbitrary files.

Treadl users can create any number of projects, and projects can be as big as you want them to be. This gives you the flexibility to use projects for the following:

* Design inspirations
* Individual weaving projects
* Scrap books
* Galleries
* Classwork
* ... and much more.

Projects can be kept private (i.e. so only you can view them), or made public. Public projects can be shared with the world via a link, so that even if someone doesn't have a Treadl account they can still view your project's contents.

Projects can also be shared to specific _groups_.

In this article we will describe the various project features and how to use them.

## Creating a new project
A project can be created by clicking the **Create a project** button from your Treadl homepage (once logged-in).

![Finding the create project button](/images/docs/projects1.png)

On the next screen you will be asked to configure your project.

![Configuring the new project](/images/docs/projects2.png)

All of this project information can be changed later.

* The project **name** is how you (and potentially others) will identify your project
* The **description** provides some extra context for the project.
* Marking a project as **private** will ensure only you can access it (unless you also add it to a group). **Non-private** (public) projects can be found and viewed by others.
* Marking a project as **open-source** will allow visitors to your project to download source WIF files and more from your project. We recommend open-source projects (just like Treadl itself is!)
* Adding your project to a **group** (or multiple groups) will allow other people in those group(s) to view your project - even if it is private.

When ready, click **Create project** to complete the creation process.

You'll now see your new project, which will be empty to start with.

![A new blank project](/images/docs/projects3.png)

## Deleting a project
To delete a project you own, click the **Settings** button on your project's page.

![Finding the settings button](/images/docs/projects6.png)

On the next screen click the **Delete project** button.

## Editing the project description
The project's description can be changed at any time. Note that you must be logged-in and you must be an owner of the project.

To do so, navigate to your project in Treadl, and then click the **Edit description** button.

![Finding the edit description button](/images/docs/projects4.png)

A text box will display, allowing you to edit the description. You can make the box bigger by dragging its corner downwards. You can now make changes to your description and then remember to click **Save description** when you've finished.

![Saving the project description](/images/docs/projects5.png)

## Changing the project's settings
In the project settings you can:

* Change the name of your project
* Update its visibility and whether it's open source or not
* Update the groups it's available to
* Delete your project

To access the project settings, click the **Settings** button on your project's page.

![Finding the settings button](/images/docs/projects6.png)

A screen will show, allowing you to change the settings (as described in "Creating a new project" above).

![The settings screen](/images/docs/projects7.png)

**Please note: project URLs (the web address in your browser) are based on the _name_ of your project. As such, if you change the name of your project the URL will also change. Bear this in mind if you have already shared a link to your project with someone.**

**Please note: if you have shared a link with someone to your project and then make your project private, they will no longer be able to access your project.**

## Adding content to your project
The whole point of having a project is that it stores patterns, items, and files! Since a project can pretty much contain anything, we call their contents _objects_.

It is quite straight forward to add objects to your project. To do so, click the **Add something** button shown on your project (if you are logged-in and own the project).

![The add something button](/images/docs/projects8.png)

From here you can:

* **Create a new weaving pattern**: this creates a new empty weaving pattern, from which you can begin to edit.
* **Import a WIF file**: this will open up your computer's file browser to allow you to choose a .wif file to upload. WIF files are special types of weaving files that Treadl can understand, and they can be created from within Treadl or other weaving software.
* **Upload an image or a file**: this will open up your computer's file browser to allow you to choose a file to upload. For example, PDFs, images, documents, and more. Image files will automatically get displayed as images in your project.

The image below displays a project now we've added a weaving pattern (by importing a WIF file) and uploaded an image.

![A project with some content](/images/docs/projects9.png)

## Viewing and editing objects
Once you have content in your project, you can click on them to view them in more detail. For example, if we click the image object we uploaded in the section above, we will see the following screen.

![Viewing an image](/images/docs/projects10.png)

On this page, a number of options are open to you.

* You can **delete** the object from the **Options** button.
* You can **rename** the object by clicking the **pencil** icon next to the name.
* You can **add a description** by clicking the **Add a description** button.
* You can add comments to your object. If your project is available to others, they can also add comments.

Adding a description to an object allows you to write in rich-text. This means you can use this tool to add commentary, research, or even an essay about your object. Descriptions are also useful for viewers

![Editing an object description](/images/docs/projects11.png)

## Viewing patterns
If you are viewing an object pattern (instead of a file or image), then other options are available to you.

![Viewing a pattern object](/images/docs/projects12.png)

You can still **delete** the pattern, add a **description** and **rename** it, but you can also:

* **Download** it as an image or in WIF format
* **Copy to** another project you own
* **Edit** the pattern in the pattern editor

The image below shows the options available when downloading a pattern.

![Downloading a pattern object](/images/docs/projects13.png)

The **download** and **copy to** options are also available to others who view your project - as long as your project is open-source.