import React, { useState, useEffect } from 'react'
import { Table, Container, Button } from 'semantic-ui-react'
import { Link } from 'react-router-dom'
import { useSelector } from 'react-redux'
import moment from 'moment'
import api from '../../../api'

function Root () {
  const [users, setUsers] = useState([])
  const [toModerate, setToModerate] = useState([])

  const { user } = useSelector(state => {
    const user = state.users.users.filter(u => state.auth.currentUserId === u._id)[0]
    return { user }
  })

  useEffect(() => {
    if (!(user?.roles?.indexOf('root') > -1)) return
    api.root.getUsers(({ users }) => {
      setUsers(users)
    })
    api.root.getToBeModerated(({ objects, comments, users }) => {
      const newToModerate = []
      objects.forEach(o => newToModerate.push({ ...o, itemType: 'objects' }))
      comments.forEach(c => newToModerate.push({ ...c, itemType: 'comments' }))
      users.forEach(u => newToModerate.push({ ...u, itemType: 'users' }))
      setToModerate(newToModerate)
    })
  }, [user])

  const testSentry = () => {
    const x = {}
    console.log(x.y.z)
  }

  const moderate = (itemType, itemId, allowed) => {
    api.root.moderate(itemType, itemId, allowed, () => {
      setToModerate(toModerate.filter(o => o._id !== itemId))
    })
  }

  return (
    <Container style={{ marginTop: '40px' }}>
      <button onClick={testSentry}>Test Sentry</button>

      <h2>Moderation</h2>
      {toModerate?.length > 0
        ? (
          <Table compact basic>
            <Table.Header>
              <Table.Row>
                <Table.Cell>Type</Table.Cell>
                <Table.Cell>Item</Table.Cell>
                <Table.Cell>Action</Table.Cell>
              </Table.Row>
            </Table.Header>
            <Table.Body>
              {toModerate.map(o =>
                <Table.Row key={o._id}>
                  <Table.Cell>{o.itemType}</Table.Cell>
                  <Table.Cell>
                    {o.itemType === 'comments' &&
                      <span>{o.content}</span>}
                    {o.itemType === 'objects' &&
                      <span>
                        <Link to={o.url} target='_blank' rel='noopener noreferrer'>{o.name}</Link>
                        {o.isImage &&
                          <img src={o.url} style={{ width: '100px' }} />}
                      </span>}
                    {o.itemType === 'users' &&
                      <span><Link to={`/${o.username}`}>{o.username}</Link></span>}
                  </Table.Cell>
                  <Table.Cell>
                    <Button onClick={() => moderate(o.itemType, o._id, true)}>Allow</Button>
                    <Button onClick={() => moderate(o.itemType, o._id, false)}>Deny</Button>
                  </Table.Cell>
                </Table.Row>
              )}
            </Table.Body>
          </Table>)
        : (
          <p>Nothing to moderate</p>
          )}

      <h2>Users</h2>
      <Table compact basic>
        <Table.Header>
          <Table.Row>
            <Table.Cell>Username</Table.Cell>
            <Table.Cell>Joined</Table.Cell>
            <Table.Cell>Last seen</Table.Cell>
            <Table.Cell>Groups</Table.Cell>
            <Table.Cell>Projects</Table.Cell>
          </Table.Row>
        </Table.Header>
        <Table.Body>
          {users.map(u =>
            <Table.Row key={u._id}>
              <Table.Cell>
                <Link to={`/${u.username}`}>{u.username}</Link> <br /> {u.email}
              </Table.Cell>
              <Table.Cell>{moment(u.createdAt).format('DD/MM/YYYY HH:mm')}</Table.Cell>
              <Table.Cell>{moment(u.lastSeenAt).format('DD/MM/YYYY HH:mm')}</Table.Cell>
              <Table.Cell>
                {u.groupMemberships?.map(g =>
                  <span key={g._id}><Link to={`/groups/${g._id}`}>{g.name}</Link><br /></span>
                )}
              </Table.Cell>
              <Table.Cell>
                {u.projects?.map(p =>
                  <span key={p._id}><Link to={`/${u.username}/${p.path}`}>{p.name}</Link><br /></span>
                )}
              </Table.Cell>
            </Table.Row>
          )}
        </Table.Body>
      </Table>
    </Container>
  )
}

export default Root
